<?php
/**
 *  2011-2020 DMConcept
 *
 *  @author    DMConcept <support@dmconcept.fr>
 *  @copyright 2020 DMConcept
 *  @license   Commercial
 */

require_once _PS_MODULE_DIR_ . 'antsroute/classes/AntsRouteManager.php';

class AdminAntsRouteConfigController extends ModuleAdminController
{
    public $name = "AdminAntsRouteConfig";

    const CONFIG_KEYS = array(
        'ANTSROUTE_CARRIER',
        'ANTSROUTE_DELIVERY_TRIGGER',
        'ANTSROUTE_CANCEL_TRIGGER',
        'ANTSROUTE_API_KEY',
        'ANTSROUTE_ADDING_ORDER',

        'ANTSROUTE_DELIVERY_WEEK_TIME_SLOTS',
        'ANTSROUTE_DELIVERY_DEADLINE',
        'ANTSROUTE_DELIVERY_TIME',


        'ANTSROUTE_ENABLE_PICKUP_DELIVERY',
        'ANTSROUTE_DEPOSIT_ADDRESS',
        'ANTSROUTE_DEPOSIT_NAME',
        'ANTSROUTE_CHARGING_TIME',
        'ANTSROUTE_CHARGING_TIME_SLOTS'
    );

    // to set the selects, this is the max values
    const TIME_MAX = 15;
    const DAY_MAX = 15;

    public function __construct()
    {
        $this->bootstrap = true;
        $this->module = 'antsroute';
        $this->lang = false;


        parent::__construct();
    }

    public function initContent()
    {
        parent::initContent();

        $this->addJS(_PS_MODULE_DIR_ . 'antsroute/views/js/admin/configuration.js');
        $this->addCSS(_PS_MODULE_DIR_ . 'antsroute/views/css/admin/configuration.css');

        $this->context->smarty->assign(
            array(
                'content' => $this->renderForm() . $this->renderAdditionalFields()
            )
        );
    }

    // to render fields that is not manage by prestashop form_fields
    public function renderAdditionalFields()
    {
        $templates = '';

        $chargingTimeSlots = json_decode(Configuration::get('ANTSROUTE_CHARGING_TIME_SLOTS'), true);
        $deliveryWeekTimeSlots = json_decode(Configuration::get('ANTSROUTE_DELIVERY_WEEK_TIME_SLOTS'), true);

        $templates .= $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . 'antsroute/views/templates/admin/fields/time_slot.tpl',
            array(
                'required' => true,
                'id' => 'ANTSROUTE_CHARGING_TIME_SLOTS',
                'name' => $this->l('Loading time slots'),
                'values' => $chargingTimeSlots,
                'hint' => $this->l('Define the time slot to which you want to load your goods.')
            )
        );
        $templates .= $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . 'antsroute/views/templates/admin/fields/week_time_slots.tpl',
            array(
                'required' => true,
                'id' => 'ANTSROUTE_DELIVERY_WEEK_TIME_SLOTS',
                'name' => $this->l('Delivery time slots'),
                'values' => $deliveryWeekTimeSlots,
                'hint' => $this->l('Define the time slots to which you want to deliver your customers.')
            )
        );

        return $templates;
    }

    public function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitAntsRouteModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminAntsRouteConfig', false).'&configure';
        $helper->token = Tools::getAdminTokenLite('AdminAntsRouteConfig');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(),
            /* Add values for your inputs */
            'languages'    => $this->context->controller->getLanguages(),
            'id_language'  => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm($helper)));
    }


    protected function getConfigForm()
    {
        $optionsStates = array();
        foreach (OrderState::getOrderStates((int) Context::getContext()->language->id) as $state) {
            $optionsStates[] = array(
                "id" => (int) $state['id_order_state'],
                "name" => $state['name']
            );
        }
        $carriers = Carrier::getCarriers(Context::getContext()->language->id, true);


        $addingOrderZones = array(
            array('id' => AntsRouteManager::INTERVENTION_BASKET, 'name' => $this->l('In the intervention basket')),
            array('id' => AntsRouteManager::MODULE_ORGANIZE, 'name' => $this->l('In the module organize'))
        );

        $deliveryTimes = array();
        $deliveryDeadlines = array();

        $chargingTimes = array();

        $booleanValues = array(
            array(
                'id'    => 'active_on',
                'value' => 1,
                'label' => $this->l('Yes')
            ),
            array(
                'id'    => 'active_off',
                'value' => 0,
                'label' => $this->l('No')
            )
        );

        for ($i = 1; $i <= self::TIME_MAX; $i++) {
            $time = array(
                'id' => $i,
                'name' => $i . ' ' .$this->l('min.')
            );
            $deliveryTimes[] = $time;
            $chargingTimes[] = $time;
        }

        for ($i = 1; $i <= self::DAY_MAX; $i++) {
            $deliveryDeadlines[] = array(
                'id' => $i,
                'name' => $this->l('D') . ' + ' . $i
            );
        }

        $generalSettings = array(
            array(
                'type'    => 'select',
                'tab'     => 'generalSettings',
                'label'   => $this->l('Choice of carrier'),
                'name'    => 'ANTSROUTE_CARRIER',
                'options' => array(
                    'query' => $carriers,
                    'id'    => 'id_reference',
                    'name'  => 'name'
                ),
                'required' => true,
                'class' => 'fixed-width-xxl',
            ),
            array(
                'type'    => 'select',
                'tab'     => 'generalSettings',
                'label'   => $this->l('Delivery initiator'),
                'name'    => 'ANTSROUTE_DELIVERY_TRIGGER',
                'options' => array(
                    'query' => $optionsStates,
                    'id'    => 'id',
                    'name'  => 'name'
                ),
                'required' => true,
                'hint' => $this->l('Define the Prestashop action that triggers a delivery.'),
                'class' => 'fixed-width-xxl',
            ),
            array(
                'type'    => 'select',
                'tab'     => 'generalSettings',
                'label'   => $this->l('Cancel initiator'),
                'name'    => 'ANTSROUTE_CANCEL_TRIGGER',
                'options' => array(
                    'query' => $optionsStates,
                    'id'    => 'id',
                    'name'  => 'name'
                ),
                'required' => true,
                'hint' => $this->l('Define the Prestashop action to cancel a delivery.'),
                'class' => 'fixed-width-xxl',
            ),
            array(
                'type'  => 'text',
                'tab'   => 'generalSettings',
                'label' => $this->l('API key'),
                'name'  => 'ANTSROUTE_API_KEY',
                'required' => true,
                'hint' => $this->l('Retrieve your API key from the AntsRoute software.'
                    .' Go to the "API key" tab of the "Last name - First name" module and click on "Add".'),
            ),
            array(
                'type'    => 'select',
                'tab'     => 'generalSettings',
                'label'   => $this->l('Adding the order'),
                'name'    => 'ANTSROUTE_ADDING_ORDER',
                'options' => array(
                    'query' => $addingOrderZones,
                    'id'    => 'id',
                    'name'  => 'name'
                ),
                'required' => true,
                'hint' => $this->l('Choose the AntsRoute module to supply with the commands.'),
                'desc' => $this->l('Delivery is planned from this tab'),
                'class' => 'fixed-width-xxl',
            ),

        );

        $deliverySettings = array(
            array(
                'type'    => 'select',
                'tab'     => 'deliverySettings',
                'label'   => $this->l('Delivery deadline'),
                'name'    => 'ANTSROUTE_DELIVERY_DEADLINE',
                'options' => array(
                    'query' => $deliveryDeadlines,
                    'id'    => 'id',
                    'name'  => 'name'
                ),
                'required' => true,
                'hint' => $this->l('Define the number of working days between the order date and the delivery date.'),
                'desc' => $this->l(
                    'The deadlines are expressed in working days with D which corresponds to the day of the order'
                ),
                'class' => 'fixed-width-xxl',
            ),
            array(
                'type'    => 'select',
                'tab'     => 'deliverySettings',
                'label'   => $this->l('Delivery time'),
                'name'    => 'ANTSROUTE_DELIVERY_TIME',
                'options' => array(
                    'query' => $deliveryTimes,
                    'id'    => 'id',
                    'name'  => 'name'
                ),
                'required' => true,
                'hint' => $this->l(
                    'Define the time required for the deliverers to deliver the goods to each customer.'
                ),
                'desc' => $this->l('time required to deliver the goods to the customer'),
                'class' => 'fixed-width-xxl',
            ),
        );

        $pickupAndDelivery = array(
            array(
                'type'   => 'switch',
                'tab'    => 'pickupAndDelivery',
                'values' => $booleanValues,
                'label'  => $this->l('Enable pick-up & delivery'),
                'name'   => 'ANTSROUTE_ENABLE_PICKUP_DELIVERY',
                'required' => true,
                'hint' => $this->l('Activate this feature if your deliverers make more than one return'
                    .' to your deposit per day (reloading the vehicle).'),
                'desc' => $this->l('If you set it to "yes", you have to activate it on the AntsRoute platform.'),
                'class' => 'fixed-width-xxl',
            ),
            array(
                'type'  => 'text',
                'tab'   => 'pickupAndDelivery',
                'label' => $this->l('Deposit address'),
                'name'  => 'ANTSROUTE_DEPOSIT_ADDRESS',
                'required' => true,
                'hint' => $this->l('Define the address of your depot, i.e. the place of loading of the goods.')
            ),
            array(
                'type'  => 'text',
                'tab'   => 'pickupAndDelivery',
                'label' => $this->l('Deposit name'),
                'name'  => 'ANTSROUTE_DEPOSIT_NAME',
                'required' => true,
                'hint' => $this->l('Define the name of your depot, i.e. the place of loading of the goods.'),
            ),
            array(
                'type'    => 'select',
                'tab'     => 'pickupAndDelivery',
                'label'   => $this->l('Loading time'),
                'name'    => 'ANTSROUTE_CHARGING_TIME',
                'options' => array(
                    'query' => $chargingTimes,
                    'id'    => 'id',
                    'name'  => 'name'
                ),
                'required' => true,
                'hint' => $this->l('Set the time required to load the goods into the vehicles.'),
                'desc' => $this->l('Time required to load the goods into the vehicle'),
                'class' => 'fixed-width-xxl',
            ),
        );

        return array(
            'form' => array(
                'legend'  => array(
                    'title' => $this->l('Settings'),
                    'icon'  => 'icon-cogs',
                ),
                'tinymce' => true,
                'tabs'    => array(
                    'generalSettings'   => '<span class="icon-gear"></span> ' . $this->l('General settings'),
                    'deliverySettings'  => '<span class="icon-ship"></span> ' . $this->l('Delivery settings'),
                    'pickupAndDelivery' => '<span class="icon-hand-left"></span> ' . $this->l('Pick-up & delivery'),
                ),
                'input'   => array_merge($generalSettings, $deliverySettings, $pickupAndDelivery),
                'submit'  => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $return = array();

        foreach (self::CONFIG_KEYS as $key) {
            $return[$key] = Configuration::get($key);
        }

        return $return;
    }

    /**
     * Save form data.
     */
    public function postProcess()
    {
        if (Tools::getValue('submitAntsRouteModule', false)) {
            foreach (self::CONFIG_KEYS as $key) {
                $value = Tools::getValue($key, null);
                if (isset($value)) {
                    if ($key === 'ANTSROUTE_CHARGING_TIME_SLOTS') {
                        Configuration::updateValue($key, json_encode($value));
                    } elseif ($key === 'ANTSROUTE_DELIVERY_WEEK_TIME_SLOTS') {
                        for ($i = 0; $i < 7; $i++) {
                            if (!isset($value[$i])) {
                                $value[$i] = array('start' => '', 'end' => '');
                            }
                        }
                        Configuration::updateValue($key, json_encode($value));
                    } else {
                        Configuration::updateValue($key, $value);
                    }
                }
            }

            $redirect_after = $this->context->link->getAdminLink('AdminAntsRouteConfig')."&conf=4";
            Tools::redirectAdmin($redirect_after);
        }
    }
}
