/**
 *  2011-2020 DMConcept
 *
 *  @author    DMConcept <support@dmconcept.fr>
 *  @copyright 2020 DMConcept
 *  @license   Commercial
 */

$.timepicker.regional['fr'] = {
    amNames: ["AM", "A"],
    ampm: false,
    closeText: "Fait",
    currentText: "Maintenant",
    hourText: "Heure",
    millisecText: "Milliseconde",
    minuteText: "Minute",
    pmNames: ["PM", "P"],
    secondText: "Seconde",
    timeFormat: "hh:mm tt",
    timeOnlyTitle: "Selection du temps",
    timeSuffix: "",
    timeText: "Temps",
    timezoneText: "Time Zone",
};

var configuration = {
    params: {
        zones: {
            pickupAndDelivery: '#pickupAndDelivery',
            deliverySettings: '#deliverySettings'
        },
        fields: {
            chargingTimeSlots: '[data-id="ANTSROUTE_CHARGING_TIME_SLOTS"]',
            deliveryWeekTimeSlots: '[data-id="ANTSROUTE_DELIVERY_WEEK_TIME_SLOTS"]'
        }
    },

    init: function () {
        this.replaceFields();
        this.initTimePicker();
        this.initEvents();
    },

    initTimePicker: function() {
        const languageIsoCode = $('#lang_iso_code').val();
        $.timepicker.setDefaults($.timepicker.regional[languageIsoCode]);
        $('.time_slot_input').timepicker({});
    },

    replaceFields: function () {
        var pickupAndDeliveryZone = $(this.params.zones.pickupAndDelivery);
        var deliverySettingsZone = $(this.params.zones.deliverySettings);

        var chargingTimeSlotsField = $(this.params.fields.chargingTimeSlots);
        var deliveryWeekTimeSlotsField = $(this.params.fields.deliveryWeekTimeSlots);

        pickupAndDeliveryZone.html(
            pickupAndDeliveryZone.html() + chargingTimeSlotsField.html()
        );
        // empty the useless zone to avoid "name override" problem
        chargingTimeSlotsField.html('');

        deliverySettingsZone.html(
            deliveryWeekTimeSlotsField.html() + deliverySettingsZone.html()
        );
        // empty the useless zone to avoid "name override" problem
        deliveryWeekTimeSlotsField.html('');
    },

    initEvents: function () {
        this.initPickupAndDeliveryEvents();
    },

    initPickupAndDeliveryEvents: function () {
        let enablePickupAndDeliveryOn =  $('#ANTSROUTE_ENABLE_PICKUP_DELIVERY_on');
        let enablePickupAndDeliveryOff =  $('#ANTSROUTE_ENABLE_PICKUP_DELIVERY_off');

        let depositAddressZone = $('#ANTSROUTE_DEPOSIT_ADDRESS').parent().parent();
        let depositNameZone = $('#ANTSROUTE_DEPOSIT_NAME').parent().parent();
        let chargingTimeZone = $('#ANTSROUTE_CHARGING_TIME').parent().parent();
        let chargingTimeSlotZone = $('#ANTSROUTE_CHARGING_TIME_SLOTS');

        if (!enablePickupAndDeliveryOn.attr('checked')) {
            depositAddressZone.hide();
            depositNameZone.hide();
            chargingTimeZone.hide();
            chargingTimeSlotZone.hide();
        }

        enablePickupAndDeliveryOn.click((e) => {
            depositAddressZone.show();
            depositNameZone.show();
            chargingTimeZone.show();
            chargingTimeSlotZone.show();
        });

        enablePickupAndDeliveryOff.click((e) => {
            depositAddressZone.hide();
            depositNameZone.hide();
            chargingTimeZone.hide();
            chargingTimeSlotZone.hide();
        });
    }

};

$(document).ready(function () {
    configuration.init();
});